﻿using ProjectTank.CollectionGenericObjects;
using ProjectTank.Exceptions;
/// <summary>
/// Параметризованный набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>

  public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
        where T : class
    {
    /// <summary>
    /// Массив объектов, которые храним
    /// </summary>
    private T?[] _collection;
    public int Count => _collection.Length;
    public int MaxCount
    {
        get
        {
            return _collection.Length;
        }
        set
        {
            if (value > 0)
            {
                if (_collection.Length > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T?[value];
                }
            }
        }
    }

    public CollectionType GetCollectionType => CollectionType.Massive;
    /// <summary>
    /// Конструктор
    /// </summary>
    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }
    public T? Get(int position)
    {
        if (position < 0 || position >= Count) throw new PositionOutOfCollectionException();
        return _collection[position];
    }
    public int Insert(T obj, IEqualityComparer<T?>? comparer = null)
    {
        if (comparer != null)
        {
            foreach (T? i in _collection)
            {
                if (comparer.Equals(i, obj))
                {
                    throw new ObjectAlreadyExistsException(i);
                }
            }
        }
        
        return Insert(obj, 0);
        throw new CollectionOverflowException();
    }
    public int Insert(T obj, int position, IEqualityComparer<T?>? comparer = null)
    {
        if (position < 0 || position >= Count)
        {
            throw new PositionOutOfCollectionException();
        }
        if (comparer != null)
        {
            foreach (T? i in _collection)
            {
                if (comparer.Equals(i, obj))
                {
                    throw new ObjectAlreadyExistsException(i);
                }
            }
        }
        if (_collection[position] == null)
        {
            _collection[position] = obj;
            return position;

        }
        int temp = position + 1;
        while (temp < Count)
        {
            if (_collection[temp] == null)
            {
                _collection[temp] = obj;
                return temp;
            }
            ++temp;
        }
        temp = position - 1;
        while (temp >= 0)
        {
            if (_collection[temp] == null)
            {
                _collection[temp] = obj;
                return temp;
            }
            --temp;
        }
        throw new CollectionOverflowException();
    }
    public T? Remove(int position)
    {
        if (position >= Count || position < 0) throw new PositionOutOfCollectionException();
        if (_collection[position] == null) throw new ObjectNotFoundException();
        T? myObject = _collection[position];
        _collection[position] = null;
        return myObject;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Length; ++i)
        {
            yield return _collection[i];
        }
    }

    public void CollectionSort(IComparer<T?> comparer)
    {
        Array.Sort(_collection, comparer);
    }
}